from dataclasses import dataclass
import uinput as k
from wayremap import ecodes as e
from wayremap import constants


@dataclass
class Binding:
    remap: list[int]
    to: list[list[tuple[int, int]]]

    def get_remap_keycode(self) -> int:
        """
        @type evdev.ecodes.KEY_F
        """
        return self.remap[len(self.remap) - 1]

    def _has_ctrl(self) -> bool:
        for k in constants.CTRL_KEYS:
            if k in self.remap:
                return True
        return False

    def _has_alt(self) -> bool:
        for k in constants.ALT_KEYS:
            if k in self.remap:
                return True
        return False

    def only_ctrl(self) -> bool:
        return self._has_ctrl() and not self._has_alt()

    def only_alt(self) -> bool:
        return not self._has_ctrl() and self._has_alt()

    def require_ctrl_alt(self) -> bool:
        return self._has_ctrl() and self._has_alt()


@dataclass
class WayremapConfig:
    applications: list[str]
    bindings: list[Binding]


example_config = WayremapConfig(
    applications=[
        'Brave-browser',
        'chromium',
        'firefoxdeveloperedition',
    ],
    bindings=[
        # Emacs-like key binding
        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_A],
                [[k.KEY_LEFTCTRL, k.KEY_HOME]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_E],
                [[k.KEY_LEFTCTRL, k.KEY_END]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_H],
                [[k.KEY_LEFTCTRL, k.KEY_BACKSPACE]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_F], [[k.KEY_RIGHT]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_B], [[k.KEY_LEFT]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_P], [[k.KEY_UP]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_N], [[k.KEY_DOWN]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_K],
                [[k.KEY_LEFTSHIFT, k.KEY_END], [k.KEY_LEFTCTRL, k.KEY_X]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_A], [[k.KEY_HOME]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_E], [[k.KEY_END]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_Y], [[k.KEY_LEFTCTRL, k.KEY_V]]),
        Binding([e.KEY_LEFTALT, e.KEY_F], [[k.KEY_LEFTCTRL, k.KEY_RIGHT]]),
        Binding([e.KEY_LEFTALT, e.KEY_B], [[k.KEY_LEFTCTRL, k.KEY_LEFT]]),
        Binding([e.KEY_LEFTALT, e.KEY_D], [[k.KEY_LEFTCTRL, k.KEY_DELETE]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_H], [[k.KEY_BACKSPACE]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_D], [[k.KEY_DELETE]]),
        Binding([e.KEY_LEFTCTRL, e.KEY_S], [[k.KEY_LEFTCTRL, k.KEY_F]]),

        # OSX-like key binding
        Binding([e.KEY_LEFTALT, e.KEY_A], [[k.KEY_LEFTCTRL, k.KEY_A]]),
        Binding([e.KEY_LEFTALT, e.KEY_C], [[k.KEY_LEFTCTRL, k.KEY_C]]),
        Binding([e.KEY_LEFTALT, e.KEY_V], [[k.KEY_LEFTCTRL, k.KEY_V]]),

        # Slack helm!
        Binding([e.KEY_LEFTALT, e.KEY_X], [[k.KEY_LEFTCTRL, k.KEY_K]]),
    ])



import evdev
from wayremap import ecodes
import uinput

CTRL_KEYS = [ecodes.KEY_CAPSLOCK, ecodes.KEY_LEFTCTRL, ecodes.KEY_RIGHTCTRL]
ALT_KEYS = [ecodes.KEY_LEFTALT, ecodes.KEY_RIGHTALT]

EV_KEY = evdev.ecodes.EV_KEY  # type: ignore

ALL_KEYS = [
    uinput.KEY_RESERVED,
    uinput.KEY_ESC,
    uinput.KEY_1,
    uinput.KEY_2,
    uinput.KEY_3,
    uinput.KEY_4,
    uinput.KEY_5,
    uinput.KEY_6,
    uinput.KEY_7,
    uinput.KEY_8,
    uinput.KEY_9,
    uinput.KEY_0,
    uinput.KEY_MINUS,
    uinput.KEY_EQUAL,
    uinput.KEY_BACKSPACE,
    uinput.KEY_TAB,
    uinput.KEY_Q,
    uinput.KEY_W,
    uinput.KEY_E,
    uinput.KEY_R,
    uinput.KEY_T,
    uinput.KEY_Y,
    uinput.KEY_U,
    uinput.KEY_I,
    uinput.KEY_O,
    uinput.KEY_P,
    uinput.KEY_LEFTBRACE,
    uinput.KEY_RIGHTBRACE,
    uinput.KEY_ENTER,
    uinput.KEY_LEFTCTRL,
    uinput.KEY_A,
    uinput.KEY_S,
    uinput.KEY_D,
    uinput.KEY_F,
    uinput.KEY_G,
    uinput.KEY_H,
    uinput.KEY_J,
    uinput.KEY_K,
    uinput.KEY_L,
    uinput.KEY_SEMICOLON,
    uinput.KEY_APOSTROPHE,
    uinput.KEY_GRAVE,
    uinput.KEY_LEFTSHIFT,
    uinput.KEY_BACKSLASH,
    uinput.KEY_Z,
    uinput.KEY_X,
    uinput.KEY_C,
    uinput.KEY_V,
    uinput.KEY_B,
    uinput.KEY_N,
    uinput.KEY_M,
    uinput.KEY_COMMA,
    uinput.KEY_DOT,
    uinput.KEY_SLASH,
    uinput.KEY_RIGHTSHIFT,
    uinput.KEY_KPASTERISK,
    uinput.KEY_LEFTALT,
    uinput.KEY_SPACE,
    uinput.KEY_CAPSLOCK,
    uinput.KEY_F1,
    uinput.KEY_F2,
    uinput.KEY_F3,
    uinput.KEY_F4,
    uinput.KEY_F5,
    uinput.KEY_F6,
    uinput.KEY_F7,
    uinput.KEY_F8,
    uinput.KEY_F9,
    uinput.KEY_F10,
    uinput.KEY_NUMLOCK,
    uinput.KEY_SCROLLLOCK,
    uinput.KEY_KP7,
    uinput.KEY_KP8,
    uinput.KEY_KP9,
    uinput.KEY_KPMINUS,
    uinput.KEY_KP4,
    uinput.KEY_KP5,
    uinput.KEY_KP6,
    uinput.KEY_KPPLUS,
    uinput.KEY_KP1,
    uinput.KEY_KP2,
    uinput.KEY_KP3,
    uinput.KEY_KP0,
    uinput.KEY_KPDOT,
    uinput.KEY_ZENKAKUHANKAKU,
    uinput.KEY_102ND,
    uinput.KEY_F11,
    uinput.KEY_F12,
    uinput.KEY_RO,
    uinput.KEY_KATAKANA,
    uinput.KEY_HIRAGANA,
    uinput.KEY_HENKAN,
    uinput.KEY_KATAKANAHIRAGANA,
    uinput.KEY_MUHENKAN,
    uinput.KEY_KPJPCOMMA,
    uinput.KEY_KPENTER,
    uinput.KEY_RIGHTCTRL,
    uinput.KEY_KPSLASH,
    uinput.KEY_SYSRQ,
    uinput.KEY_RIGHTALT,
    uinput.KEY_LINEFEED,
    uinput.KEY_HOME,
    uinput.KEY_UP,
    uinput.KEY_PAGEUP,
    uinput.KEY_LEFT,
    uinput.KEY_RIGHT,
    uinput.KEY_END,
    uinput.KEY_DOWN,
    uinput.KEY_PAGEDOWN,
    uinput.KEY_INSERT,
    uinput.KEY_DELETE,
    uinput.KEY_MACRO,
    uinput.KEY_MUTE,
    uinput.KEY_VOLUMEDOWN,
    uinput.KEY_VOLUMEUP,
    uinput.KEY_POWER,
    uinput.KEY_KPEQUAL,
    uinput.KEY_KPPLUSMINUS,
    uinput.KEY_PAUSE,
    uinput.KEY_SCALE,
    uinput.KEY_KPCOMMA,
    uinput.KEY_HANGEUL,
    uinput.KEY_HANGUEL,
    uinput.KEY_HANJA,
    uinput.KEY_YEN,
    uinput.KEY_LEFTMETA,
    uinput.KEY_RIGHTMETA,
    uinput.KEY_COMPOSE,
    uinput.KEY_STOP,
    uinput.KEY_AGAIN,
    uinput.KEY_PROPS,
    uinput.KEY_UNDO,
    uinput.KEY_FRONT,
    uinput.KEY_COPY,
    uinput.KEY_OPEN,
    uinput.KEY_PASTE,
    uinput.KEY_FIND,
    uinput.KEY_CUT,
    uinput.KEY_HELP,
    uinput.KEY_MENU,
    uinput.KEY_CALC,
    uinput.KEY_SETUP,
    uinput.KEY_SLEEP,
    uinput.KEY_WAKEUP,
    uinput.KEY_FILE,
    uinput.KEY_SENDFILE,
    uinput.KEY_DELETEFILE,
    uinput.KEY_XFER,
    uinput.KEY_PROG1,
    uinput.KEY_PROG2,
    uinput.KEY_WWW,
    uinput.KEY_MSDOS,
    uinput.KEY_COFFEE,
    uinput.KEY_SCREENLOCK,
    uinput.KEY_ROTATE_DISPLAY,
    uinput.KEY_DIRECTION,
    uinput.KEY_CYCLEWINDOWS,
    uinput.KEY_MAIL,
    uinput.KEY_BOOKMARKS,
    uinput.KEY_COMPUTER,
    uinput.KEY_BACK,
    uinput.KEY_FORWARD,
    uinput.KEY_CLOSECD,
    uinput.KEY_EJECTCD,
    uinput.KEY_EJECTCLOSECD,
    uinput.KEY_NEXTSONG,
    uinput.KEY_PLAYPAUSE,
    uinput.KEY_PREVIOUSSONG,
    uinput.KEY_STOPCD,
    uinput.KEY_RECORD,
    uinput.KEY_REWIND,
    uinput.KEY_PHONE,
    uinput.KEY_ISO,
    uinput.KEY_CONFIG,
    uinput.KEY_HOMEPAGE,
    uinput.KEY_REFRESH,
    uinput.KEY_EXIT,
    uinput.KEY_MOVE,
    uinput.KEY_EDIT,
    uinput.KEY_SCROLLUP,
    uinput.KEY_SCROLLDOWN,
    uinput.KEY_KPLEFTPAREN,
    uinput.KEY_KPRIGHTPAREN,
    uinput.KEY_NEW,
    uinput.KEY_REDO,
    uinput.KEY_F13,
    uinput.KEY_F14,
    uinput.KEY_F15,
    uinput.KEY_F16,
    uinput.KEY_F17,
    uinput.KEY_F18,
    uinput.KEY_F19,
    uinput.KEY_F20,
    uinput.KEY_F21,
    uinput.KEY_F22,
    uinput.KEY_F23,
    uinput.KEY_F24,
    uinput.KEY_PLAYCD,
    uinput.KEY_PAUSECD,
    uinput.KEY_PROG3,
    uinput.KEY_PROG4,
    uinput.KEY_DASHBOARD,
    uinput.KEY_SUSPEND,
    uinput.KEY_CLOSE,
    uinput.KEY_PLAY,
    uinput.KEY_FASTFORWARD,
    uinput.KEY_BASSBOOST,
    uinput.KEY_PRINT,
    uinput.KEY_HP,
    uinput.KEY_CAMERA,
    uinput.KEY_SOUND,
    uinput.KEY_QUESTION,
    uinput.KEY_EMAIL,
    uinput.KEY_CHAT,
    uinput.KEY_SEARCH,
    uinput.KEY_CONNECT,
    uinput.KEY_FINANCE,
    uinput.KEY_SPORT,
    uinput.KEY_SHOP,
    uinput.KEY_ALTERASE,
    uinput.KEY_CANCEL,
    uinput.KEY_BRIGHTNESSDOWN,
    uinput.KEY_BRIGHTNESSUP,
    uinput.KEY_MEDIA,
    uinput.KEY_SWITCHVIDEOMODE,
    uinput.KEY_KBDILLUMTOGGLE,
    uinput.KEY_KBDILLUMDOWN,
    uinput.KEY_KBDILLUMUP,
    uinput.KEY_SEND,
    uinput.KEY_REPLY,
    uinput.KEY_FORWARDMAIL,
    uinput.KEY_SAVE,
    uinput.KEY_DOCUMENTS,
    uinput.KEY_BATTERY,
    uinput.KEY_BLUETOOTH,
    uinput.KEY_WLAN,
    uinput.KEY_UWB,
    uinput.KEY_UNKNOWN,
    uinput.KEY_VIDEO_NEXT,
    uinput.KEY_VIDEO_PREV,
    uinput.KEY_BRIGHTNESS_CYCLE,
    uinput.KEY_BRIGHTNESS_AUTO,
    uinput.KEY_BRIGHTNESS_ZERO,
    uinput.KEY_DISPLAY_OFF,
    uinput.KEY_WWAN,
    uinput.KEY_WIMAX,
    uinput.KEY_RFKILL,
    uinput.KEY_MICMUTE,
    uinput.KEY_OK,
    uinput.KEY_SELECT,
    uinput.KEY_GOTO,
    uinput.KEY_CLEAR,
    uinput.KEY_POWER2,
    uinput.KEY_OPTION,
    uinput.KEY_INFO,
    uinput.KEY_TIME,
    uinput.KEY_VENDOR,
    uinput.KEY_ARCHIVE,
    uinput.KEY_PROGRAM,
    uinput.KEY_CHANNEL,
    uinput.KEY_FAVORITES,
    uinput.KEY_EPG,
    uinput.KEY_PVR,
    uinput.KEY_MHP,
    uinput.KEY_LANGUAGE,
    uinput.KEY_TITLE,
    uinput.KEY_SUBTITLE,
    uinput.KEY_ANGLE,
    uinput.KEY_ZOOM,
    uinput.KEY_MODE,
    uinput.KEY_KEYBOARD,
    uinput.KEY_SCREEN,
    uinput.KEY_PC,
    uinput.KEY_TV,
    uinput.KEY_TV2,
    uinput.KEY_VCR,
    uinput.KEY_VCR2,
    uinput.KEY_SAT,
    uinput.KEY_SAT2,
    uinput.KEY_CD,
    uinput.KEY_TAPE,
    uinput.KEY_RADIO,
    uinput.KEY_TUNER,
    uinput.KEY_PLAYER,
    uinput.KEY_TEXT,
    uinput.KEY_DVD,
    uinput.KEY_AUX,
    uinput.KEY_MP3,
    uinput.KEY_AUDIO,
    uinput.KEY_VIDEO,
    uinput.KEY_DIRECTORY,
    uinput.KEY_LIST,
    uinput.KEY_MEMO,
    uinput.KEY_CALENDAR,
    uinput.KEY_RED,
    uinput.KEY_GREEN,
    uinput.KEY_YELLOW,
    uinput.KEY_BLUE,
    uinput.KEY_CHANNELUP,
    uinput.KEY_CHANNELDOWN,
    uinput.KEY_FIRST,
    uinput.KEY_LAST,
    uinput.KEY_AB,
    uinput.KEY_NEXT,
    uinput.KEY_RESTART,
    uinput.KEY_SLOW,
    uinput.KEY_SHUFFLE,
    uinput.KEY_BREAK,
    uinput.KEY_PREVIOUS,
    uinput.KEY_DIGITS,
    uinput.KEY_TEEN,
    uinput.KEY_TWEN,
    uinput.KEY_VIDEOPHONE,
    uinput.KEY_GAMES,
    uinput.KEY_ZOOMIN,
    uinput.KEY_ZOOMOUT,
    uinput.KEY_ZOOMRESET,
    uinput.KEY_WORDPROCESSOR,
    uinput.KEY_EDITOR,
    uinput.KEY_SPREADSHEET,
    uinput.KEY_GRAPHICSEDITOR,
    uinput.KEY_PRESENTATION,
    uinput.KEY_DATABASE,
    uinput.KEY_NEWS,
    uinput.KEY_VOICEMAIL,
    uinput.KEY_ADDRESSBOOK,
    uinput.KEY_MESSENGER,
    uinput.KEY_DISPLAYTOGGLE,
    uinput.KEY_BRIGHTNESS_TOGGLE,
    uinput.KEY_SPELLCHECK,
    uinput.KEY_LOGOFF,
    uinput.KEY_DOLLAR,
    uinput.KEY_EURO,
    uinput.KEY_FRAMEBACK,
    uinput.KEY_FRAMEFORWARD,
    uinput.KEY_CONTEXT_MENU,
    uinput.KEY_MEDIA_REPEAT,
    uinput.KEY_10CHANNELSUP,
    uinput.KEY_10CHANNELSDOWN,
    uinput.KEY_IMAGES,
    uinput.KEY_DEL_EOL,
    uinput.KEY_DEL_EOS,
    uinput.KEY_INS_LINE,
    uinput.KEY_DEL_LINE,
    uinput.KEY_FN,
    uinput.KEY_FN_ESC,
    uinput.KEY_FN_F1,
    uinput.KEY_FN_F2,
    uinput.KEY_FN_F3,
    uinput.KEY_FN_F4,
    uinput.KEY_FN_F5,
    uinput.KEY_FN_F6,
    uinput.KEY_FN_F7,
    uinput.KEY_FN_F8,
    uinput.KEY_FN_F9,
    uinput.KEY_FN_F10,
    uinput.KEY_FN_F11,
    uinput.KEY_FN_F12,
    uinput.KEY_FN_1,
    uinput.KEY_FN_2,
    uinput.KEY_FN_D,
    uinput.KEY_FN_E,
    uinput.KEY_FN_F,
    uinput.KEY_FN_S,
    uinput.KEY_FN_B,
    uinput.KEY_BRL_DOT1,
    uinput.KEY_BRL_DOT2,
    uinput.KEY_BRL_DOT3,
    uinput.KEY_BRL_DOT4,
    uinput.KEY_BRL_DOT5,
    uinput.KEY_BRL_DOT6,
    uinput.KEY_BRL_DOT7,
    uinput.KEY_BRL_DOT8,
    uinput.KEY_BRL_DOT9,
    uinput.KEY_BRL_DOT10,
    uinput.KEY_NUMERIC_0,
    uinput.KEY_NUMERIC_1,
    uinput.KEY_NUMERIC_2,
    uinput.KEY_NUMERIC_3,
    uinput.KEY_NUMERIC_4,
    uinput.KEY_NUMERIC_5,
    uinput.KEY_NUMERIC_6,
    uinput.KEY_NUMERIC_7,
    uinput.KEY_NUMERIC_8,
    uinput.KEY_NUMERIC_9,
    uinput.KEY_NUMERIC_STAR,
    uinput.KEY_NUMERIC_POUND,
    uinput.KEY_NUMERIC_A,
    uinput.KEY_NUMERIC_B,
    uinput.KEY_NUMERIC_C,
    uinput.KEY_NUMERIC_D,
    uinput.KEY_CAMERA_FOCUS,
    uinput.KEY_WPS_BUTTON,
    uinput.KEY_TOUCHPAD_TOGGLE,
    uinput.KEY_TOUCHPAD_ON,
    uinput.KEY_TOUCHPAD_OFF,
    uinput.KEY_CAMERA_ZOOMIN,
    uinput.KEY_CAMERA_ZOOMOUT,
    uinput.KEY_CAMERA_UP,
    uinput.KEY_CAMERA_DOWN,
    uinput.KEY_CAMERA_LEFT,
    uinput.KEY_CAMERA_RIGHT,
    uinput.KEY_ATTENDANT_ON,
    uinput.KEY_ATTENDANT_OFF,
    uinput.KEY_ATTENDANT_TOGGLE,
    uinput.KEY_LIGHTS_TOGGLE,
    uinput.KEY_ALS_TOGGLE,
    uinput.KEY_BUTTONCONFIG,
    uinput.KEY_TASKMANAGER,
    uinput.KEY_JOURNAL,
    uinput.KEY_CONTROLPANEL,
    uinput.KEY_APPSELECT,
    uinput.KEY_SCREENSAVER,
    uinput.KEY_VOICECOMMAND,
    uinput.KEY_BRIGHTNESS_MIN,
    uinput.KEY_BRIGHTNESS_MAX,
    uinput.KEY_KBDINPUTASSIST_PREV,
    uinput.KEY_KBDINPUTASSIST_NEXT,
    uinput.KEY_KBDINPUTASSIST_PREVGROUP,
    uinput.KEY_KBDINPUTASSIST_NEXTGROUP,
    uinput.KEY_KBDINPUTASSIST_ACCEPT,
    uinput.KEY_KBDINPUTASSIST_CANCEL,
    uinput.KEY_MIN_INTERESTING,
    uinput.KEY_MAX,
]



from i3ipc import Connection, Event

i3 = Connection()


# Dynamically name your workspaces after the current window class
def on_window_focus(i3, _):
    focused = i3.get_tree().find_focused()
    app_name = focused.app_id or focused.window_class
    print(app_name)


# Subscribe to events
i3.on(Event.WINDOW_FOCUS, on_window_focus)

# Start the main loop and wait for events to come in.
i3.main()



import os


def find(name, path):
    for root, _, files in os.walk(path):
        for file in files:
            if 'sway-ipc' in file:
                print(file)
                return os.path.join(root, name)


print(find('sway-ipc.1000.1601.sock', '/run/user/'))



"""
HACK: this file is ugly, but necessary for pyright work
"""

import evdev.ecodes as e

# HACK: To declare as int, declare them as 1
KEY_0 = e.KEY_0  # type:ignore
KEY_1 = e.KEY_1  # type:ignore
KEY_102ND = e.KEY_102ND  # type:ignore
KEY_10CHANNELSDOWN = e.KEY_10CHANNELSDOWN  # type:ignore
KEY_10CHANNELSUP = e.KEY_10CHANNELSUP  # type:ignore
KEY_2 = e.KEY_2  # type:ignore
KEY_3 = e.KEY_3  # type:ignore
KEY_3D_MODE = e.KEY_3D_MODE  # type:ignore
KEY_4 = e.KEY_4  # type:ignore
KEY_5 = e.KEY_5  # type:ignore
KEY_6 = e.KEY_6  # type:ignore
KEY_7 = e.KEY_7  # type:ignore
KEY_8 = e.KEY_8  # type:ignore
KEY_9 = e.KEY_9  # type:ignore
KEY_A = e.KEY_A  # type:ignore
KEY_AB = e.KEY_AB  # type:ignore
KEY_ADDRESSBOOK = e.KEY_ADDRESSBOOK  # type:ignore
KEY_AGAIN = e.KEY_AGAIN  # type:ignore
KEY_ALS_TOGGLE = e.KEY_ALS_TOGGLE  # type:ignore
KEY_ALTERASE = e.KEY_ALTERASE  # type:ignore
KEY_ANGLE = e.KEY_ANGLE  # type:ignore
KEY_APOSTROPHE = e.KEY_APOSTROPHE  # type:ignore
KEY_APPSELECT = e.KEY_APPSELECT  # type:ignore
KEY_ARCHIVE = e.KEY_ARCHIVE  # type:ignore
KEY_ASPECT_RATIO = e.KEY_ASPECT_RATIO  # type:ignore
KEY_ASSISTANT = e.KEY_ASSISTANT  # type:ignore
KEY_ATTENDANT_OFF = e.KEY_ATTENDANT_OFF  # type:ignore
KEY_ATTENDANT_ON = e.KEY_ATTENDANT_ON  # type:ignore
KEY_ATTENDANT_TOGGLE = e.KEY_ATTENDANT_TOGGLE  # type:ignore
KEY_AUDIO = e.KEY_AUDIO  # type:ignore
KEY_AUDIO_DESC = e.KEY_AUDIO_DESC  # type:ignore
KEY_AUX = e.KEY_AUX  # type:ignore
KEY_B = e.KEY_B  # type:ignore
KEY_BACK = e.KEY_BACK  # type:ignore
KEY_BACKSLASH = e.KEY_BACKSLASH  # type:ignore
KEY_BACKSPACE = e.KEY_BACKSPACE  # type:ignore
KEY_BASSBOOST = e.KEY_BASSBOOST  # type:ignore
KEY_BATTERY = e.KEY_BATTERY  # type:ignore
KEY_BLUE = e.KEY_BLUE  # type:ignore
KEY_BLUETOOTH = e.KEY_BLUETOOTH  # type:ignore
KEY_BOOKMARKS = e.KEY_BOOKMARKS  # type:ignore
KEY_BREAK = e.KEY_BREAK  # type:ignore
KEY_BRIGHTNESSDOWN = e.KEY_BRIGHTNESSDOWN  # type:ignore
KEY_BRIGHTNESSUP = e.KEY_BRIGHTNESSUP  # type:ignore
KEY_BRIGHTNESS_AUTO = e.KEY_BRIGHTNESS_AUTO  # type:ignore
KEY_BRIGHTNESS_CYCLE = e.KEY_BRIGHTNESS_CYCLE  # type:ignore
KEY_BRIGHTNESS_MAX = e.KEY_BRIGHTNESS_MAX  # type:ignore
KEY_BRIGHTNESS_MIN = e.KEY_BRIGHTNESS_MIN  # type:ignore
KEY_BRIGHTNESS_TOGGLE = e.KEY_BRIGHTNESS_TOGGLE  # type:ignore
KEY_BRIGHTNESS_ZERO = e.KEY_BRIGHTNESS_ZERO  # type:ignore
KEY_BRL_DOT1 = e.KEY_BRL_DOT1  # type:ignore
KEY_BRL_DOT10 = e.KEY_BRL_DOT10  # type:ignore
KEY_BRL_DOT2 = e.KEY_BRL_DOT2  # type:ignore
KEY_BRL_DOT3 = e.KEY_BRL_DOT3  # type:ignore
KEY_BRL_DOT4 = e.KEY_BRL_DOT4  # type:ignore
KEY_BRL_DOT5 = e.KEY_BRL_DOT5  # type:ignore
KEY_BRL_DOT6 = e.KEY_BRL_DOT6  # type:ignore
KEY_BRL_DOT7 = e.KEY_BRL_DOT7  # type:ignore
KEY_BRL_DOT8 = e.KEY_BRL_DOT8  # type:ignore
KEY_BRL_DOT9 = e.KEY_BRL_DOT9  # type:ignore
KEY_BUTTONCONFIG = e.KEY_BUTTONCONFIG  # type:ignore
KEY_C = e.KEY_C  # type:ignore
KEY_CALC = e.KEY_CALC  # type:ignore
KEY_CALENDAR = e.KEY_CALENDAR  # type:ignore
KEY_CAMERA = e.KEY_CAMERA  # type:ignore
KEY_CAMERA_DOWN = e.KEY_CAMERA_DOWN  # type:ignore
KEY_CAMERA_FOCUS = e.KEY_CAMERA_FOCUS  # type:ignore
KEY_CAMERA_LEFT = e.KEY_CAMERA_LEFT  # type:ignore
KEY_CAMERA_RIGHT = e.KEY_CAMERA_RIGHT  # type:ignore
KEY_CAMERA_UP = e.KEY_CAMERA_UP  # type:ignore
KEY_CAMERA_ZOOMIN = e.KEY_CAMERA_ZOOMIN  # type:ignore
KEY_CAMERA_ZOOMOUT = e.KEY_CAMERA_ZOOMOUT  # type:ignore
KEY_CANCEL = e.KEY_CANCEL  # type:ignore
KEY_CAPSLOCK = e.KEY_CAPSLOCK  # type:ignore
KEY_CD = e.KEY_CD  # type:ignore
KEY_CHANNEL = e.KEY_CHANNEL  # type:ignore
KEY_CHANNELDOWN = e.KEY_CHANNELDOWN  # type:ignore
KEY_CHANNELUP = e.KEY_CHANNELUP  # type:ignore
KEY_CHAT = e.KEY_CHAT  # type:ignore
KEY_CLEAR = e.KEY_CLEAR  # type:ignore
KEY_CLOSE = e.KEY_CLOSE  # type:ignore
KEY_CLOSECD = e.KEY_CLOSECD  # type:ignore
KEY_CNT = e.KEY_CNT  # type:ignore
KEY_COFFEE = e.KEY_COFFEE  # type:ignore
KEY_COMMA = e.KEY_COMMA  # type:ignore
KEY_COMPOSE = e.KEY_COMPOSE  # type:ignore
KEY_COMPUTER = e.KEY_COMPUTER  # type:ignore
KEY_CONFIG = e.KEY_CONFIG  # type:ignore
KEY_CONNECT = e.KEY_CONNECT  # type:ignore
KEY_CONTEXT_MENU = e.KEY_CONTEXT_MENU  # type:ignore
KEY_CONTROLPANEL = e.KEY_CONTROLPANEL  # type:ignore
KEY_COPY = e.KEY_COPY  # type:ignore
KEY_CUT = e.KEY_CUT  # type:ignore
KEY_CYCLEWINDOWS = e.KEY_CYCLEWINDOWS  # type:ignore
KEY_D = e.KEY_D  # type:ignore
KEY_DASHBOARD = e.KEY_DASHBOARD  # type:ignore
KEY_DATA = e.KEY_DATA  # type:ignore
KEY_DATABASE = e.KEY_DATABASE  # type:ignore
KEY_DELETE = e.KEY_DELETE  # type:ignore
KEY_DELETEFILE = e.KEY_DELETEFILE  # type:ignore
KEY_DEL_EOL = e.KEY_DEL_EOL  # type:ignore
KEY_DEL_EOS = e.KEY_DEL_EOS  # type:ignore
KEY_DEL_LINE = e.KEY_DEL_LINE  # type:ignore
KEY_DIGITS = e.KEY_DIGITS  # type:ignore
KEY_DIRECTION = e.KEY_DIRECTION  # type:ignore
KEY_DIRECTORY = e.KEY_DIRECTORY  # type:ignore
KEY_DISPLAYTOGGLE = e.KEY_DISPLAYTOGGLE  # type:ignore
KEY_DISPLAY_OFF = e.KEY_DISPLAY_OFF  # type:ignore
KEY_DOCUMENTS = e.KEY_DOCUMENTS  # type:ignore
KEY_DOLLAR = e.KEY_DOLLAR  # type:ignore
KEY_DOT = e.KEY_DOT  # type:ignore
KEY_DOWN = e.KEY_DOWN  # type:ignore
KEY_DVD = e.KEY_DVD  # type:ignore
KEY_E = e.KEY_E  # type:ignore
KEY_EDIT = e.KEY_EDIT  # type:ignore
KEY_EDITOR = e.KEY_EDITOR  # type:ignore
KEY_EJECTCD = e.KEY_EJECTCD  # type:ignore
KEY_EJECTCLOSECD = e.KEY_EJECTCLOSECD  # type:ignore
KEY_EMAIL = e.KEY_EMAIL  # type:ignore
KEY_END = e.KEY_END  # type:ignore
KEY_ENTER = e.KEY_ENTER  # type:ignore
KEY_EPG = e.KEY_EPG  # type:ignore
KEY_EQUAL = e.KEY_EQUAL  # type:ignore
KEY_ESC = e.KEY_ESC  # type:ignore
KEY_EURO = e.KEY_EURO  # type:ignore
KEY_EXIT = e.KEY_EXIT  # type:ignore
KEY_F = e.KEY_F  # type:ignore
KEY_F1 = e.KEY_F1  # type:ignore
KEY_F10 = e.KEY_F10  # type:ignore
KEY_F11 = e.KEY_F11  # type:ignore
KEY_F12 = e.KEY_F12  # type:ignore
KEY_F13 = e.KEY_F13  # type:ignore
KEY_F14 = e.KEY_F14  # type:ignore
KEY_F15 = e.KEY_F15  # type:ignore
KEY_F16 = e.KEY_F16  # type:ignore
KEY_F17 = e.KEY_F17  # type:ignore
KEY_F18 = e.KEY_F18  # type:ignore
KEY_F19 = e.KEY_F19  # type:ignore
KEY_F2 = e.KEY_F2  # type:ignore
KEY_F20 = e.KEY_F20  # type:ignore
KEY_F21 = e.KEY_F21  # type:ignore
KEY_F22 = e.KEY_F22  # type:ignore
KEY_F23 = e.KEY_F23  # type:ignore
KEY_F24 = e.KEY_F24  # type:ignore
KEY_F3 = e.KEY_F3  # type:ignore
KEY_F4 = e.KEY_F4  # type:ignore
KEY_F5 = e.KEY_F5  # type:ignore
KEY_F6 = e.KEY_F6  # type:ignore
KEY_F7 = e.KEY_F7  # type:ignore
KEY_F8 = e.KEY_F8  # type:ignore
KEY_F9 = e.KEY_F9  # type:ignore
KEY_FASTFORWARD = e.KEY_FASTFORWARD  # type:ignore
KEY_FASTREVERSE = e.KEY_FASTREVERSE  # type:ignore
KEY_FAVORITES = e.KEY_FAVORITES  # type:ignore
KEY_FILE = e.KEY_FILE  # type:ignore
KEY_FINANCE = e.KEY_FINANCE  # type:ignore
KEY_FIND = e.KEY_FIND  # type:ignore
KEY_FIRST = e.KEY_FIRST  # type:ignore
KEY_FN = e.KEY_FN  # type:ignore
KEY_FN_1 = e.KEY_FN_1  # type:ignore
KEY_FN_2 = e.KEY_FN_2  # type:ignore
KEY_FN_B = e.KEY_FN_B  # type:ignore
KEY_FN_D = e.KEY_FN_D  # type:ignore
KEY_FN_E = e.KEY_FN_E  # type:ignore
KEY_FN_ESC = e.KEY_FN_ESC  # type:ignore
KEY_FN_F = e.KEY_FN_F  # type:ignore
KEY_FN_F1 = e.KEY_FN_F1  # type:ignore
KEY_FN_F10 = e.KEY_FN_F10  # type:ignore
KEY_FN_F11 = e.KEY_FN_F11  # type:ignore
KEY_FN_F12 = e.KEY_FN_F12  # type:ignore
KEY_FN_F2 = e.KEY_FN_F2  # type:ignore
KEY_FN_F3 = e.KEY_FN_F3  # type:ignore
KEY_FN_F4 = e.KEY_FN_F4  # type:ignore
KEY_FN_F5 = e.KEY_FN_F5  # type:ignore
KEY_FN_F6 = e.KEY_FN_F6  # type:ignore
KEY_FN_F7 = e.KEY_FN_F7  # type:ignore
KEY_FN_F8 = e.KEY_FN_F8  # type:ignore
KEY_FN_F9 = e.KEY_FN_F9  # type:ignore
KEY_FN_RIGHT_SHIFT = e.KEY_FN_RIGHT_SHIFT  # type:ignore
KEY_FN_S = e.KEY_FN_S  # type:ignore
KEY_FORWARD = e.KEY_FORWARD  # type:ignore
KEY_FORWARDMAIL = e.KEY_FORWARDMAIL  # type:ignore
KEY_FRAMEBACK = e.KEY_FRAMEBACK  # type:ignore
KEY_FRAMEFORWARD = e.KEY_FRAMEFORWARD  # type:ignore
KEY_FRONT = e.KEY_FRONT  # type:ignore
KEY_FULL_SCREEN = e.KEY_FULL_SCREEN  # type:ignore
KEY_G = e.KEY_G  # type:ignore
KEY_GAMES = e.KEY_GAMES  # type:ignore
KEY_GOTO = e.KEY_GOTO  # type:ignore
KEY_GRAPHICSEDITOR = e.KEY_GRAPHICSEDITOR  # type:ignore
KEY_GRAVE = e.KEY_GRAVE  # type:ignore
KEY_GREEN = e.KEY_GREEN  # type:ignore
KEY_H = e.KEY_H  # type:ignore
KEY_HANGEUL = e.KEY_HANGEUL  # type:ignore
KEY_HANGUEL = e.KEY_HANGUEL  # type:ignore
KEY_HANGUP_PHONE = e.KEY_HANGUP_PHONE  # type:ignore
KEY_HANJA = e.KEY_HANJA  # type:ignore
KEY_HELP = e.KEY_HELP  # type:ignore
KEY_HENKAN = e.KEY_HENKAN  # type:ignore
KEY_HIRAGANA = e.KEY_HIRAGANA  # type:ignore
KEY_HOME = e.KEY_HOME  # type:ignore
KEY_HOMEPAGE = e.KEY_HOMEPAGE  # type:ignore
KEY_HP = e.KEY_HP  # type:ignore
KEY_I = e.KEY_I  # type:ignore
KEY_IMAGES = e.KEY_IMAGES  # type:ignore
KEY_INFO = e.KEY_INFO  # type:ignore
KEY_INSERT = e.KEY_INSERT  # type:ignore
KEY_INS_LINE = e.KEY_INS_LINE  # type:ignore
KEY_ISO = e.KEY_ISO  # type:ignore
KEY_J = e.KEY_J  # type:ignore
KEY_JOURNAL = e.KEY_JOURNAL  # type:ignore
KEY_K = e.KEY_K  # type:ignore
KEY_KATAKANA = e.KEY_KATAKANA  # type:ignore
KEY_KATAKANAHIRAGANA = e.KEY_KATAKANAHIRAGANA  # type:ignore
KEY_KBDILLUMDOWN = e.KEY_KBDILLUMDOWN  # type:ignore
KEY_KBDILLUMTOGGLE = e.KEY_KBDILLUMTOGGLE  # type:ignore
KEY_KBDILLUMUP = e.KEY_KBDILLUMUP  # type:ignore
KEY_KBDINPUTASSIST_ACCEPT = e.KEY_KBDINPUTASSIST_ACCEPT  # type:ignore
KEY_KBDINPUTASSIST_CANCEL = e.KEY_KBDINPUTASSIST_CANCEL  # type:ignore
KEY_KBDINPUTASSIST_NEXT = e.KEY_KBDINPUTASSIST_NEXT  # type:ignore
KEY_KBDINPUTASSIST_NEXTGROUP = e.KEY_KBDINPUTASSIST_NEXTGROUP  # type:ignore
KEY_KBDINPUTASSIST_PREV = e.KEY_KBDINPUTASSIST_PREV  # type:ignore
KEY_KBDINPUTASSIST_PREVGROUP = e.KEY_KBDINPUTASSIST_PREVGROUP  # type:ignore
KEY_KBD_LAYOUT_NEXT = e.KEY_KBD_LAYOUT_NEXT  # type:ignore
KEY_KBD_LCD_MENU1 = e.KEY_KBD_LCD_MENU1  # type:ignore
KEY_KBD_LCD_MENU2 = e.KEY_KBD_LCD_MENU2  # type:ignore
KEY_KBD_LCD_MENU3 = e.KEY_KBD_LCD_MENU3  # type:ignore
KEY_KBD_LCD_MENU4 = e.KEY_KBD_LCD_MENU4  # type:ignore
KEY_KBD_LCD_MENU5 = e.KEY_KBD_LCD_MENU5  # type:ignore
KEY_KEYBOARD = e.KEY_KEYBOARD  # type:ignore
KEY_KP0 = e.KEY_KP0  # type:ignore
KEY_KP1 = e.KEY_KP1  # type:ignore
KEY_KP2 = e.KEY_KP2  # type:ignore
KEY_KP3 = e.KEY_KP3  # type:ignore
KEY_KP4 = e.KEY_KP4  # type:ignore
KEY_KP5 = e.KEY_KP5  # type:ignore
KEY_KP6 = e.KEY_KP6  # type:ignore
KEY_KP7 = e.KEY_KP7  # type:ignore
KEY_KP8 = e.KEY_KP8  # type:ignore
KEY_KP9 = e.KEY_KP9  # type:ignore
KEY_KPASTERISK = e.KEY_KPASTERISK  # type:ignore
KEY_KPCOMMA = e.KEY_KPCOMMA  # type:ignore
KEY_KPDOT = e.KEY_KPDOT  # type:ignore
KEY_KPENTER = e.KEY_KPENTER  # type:ignore
KEY_KPEQUAL = e.KEY_KPEQUAL  # type:ignore
KEY_KPJPCOMMA = e.KEY_KPJPCOMMA  # type:ignore
KEY_KPLEFTPAREN = e.KEY_KPLEFTPAREN  # type:ignore
KEY_KPMINUS = e.KEY_KPMINUS  # type:ignore
KEY_KPPLUS = e.KEY_KPPLUS  # type:ignore
KEY_KPPLUSMINUS = e.KEY_KPPLUSMINUS  # type:ignore
KEY_KPRIGHTPAREN = e.KEY_KPRIGHTPAREN  # type:ignore
KEY_KPSLASH = e.KEY_KPSLASH  # type:ignore
KEY_L = e.KEY_L  # type:ignore
KEY_LANGUAGE = e.KEY_LANGUAGE  # type:ignore
KEY_LAST = e.KEY_LAST  # type:ignore
KEY_LEFT = e.KEY_LEFT  # type:ignore
KEY_LEFTALT = e.KEY_LEFTALT  # type:ignore
KEY_LEFTBRACE = e.KEY_LEFTBRACE  # type:ignore
KEY_LEFTCTRL = e.KEY_LEFTCTRL  # type:ignore
KEY_LEFTMETA = e.KEY_LEFTMETA  # type:ignore
KEY_LEFTSHIFT = e.KEY_LEFTSHIFT  # type:ignore
KEY_LEFT_DOWN = e.KEY_LEFT_DOWN  # type:ignore
KEY_LEFT_UP = e.KEY_LEFT_UP  # type:ignore
KEY_LIGHTS_TOGGLE = e.KEY_LIGHTS_TOGGLE  # type:ignore
KEY_LINEFEED = e.KEY_LINEFEED  # type:ignore
KEY_LIST = e.KEY_LIST  # type:ignore
KEY_LOGOFF = e.KEY_LOGOFF  # type:ignore
KEY_M = e.KEY_M  # type:ignore
KEY_MACRO = e.KEY_MACRO  # type:ignore
KEY_MACRO1 = e.KEY_MACRO1  # type:ignore
KEY_MACRO10 = e.KEY_MACRO10  # type:ignore
KEY_MACRO11 = e.KEY_MACRO11  # type:ignore
KEY_MACRO12 = e.KEY_MACRO12  # type:ignore
KEY_MACRO13 = e.KEY_MACRO13  # type:ignore
KEY_MACRO14 = e.KEY_MACRO14  # type:ignore
KEY_MACRO15 = e.KEY_MACRO15  # type:ignore
KEY_MACRO16 = e.KEY_MACRO16  # type:ignore
KEY_MACRO17 = e.KEY_MACRO17  # type:ignore
KEY_MACRO18 = e.KEY_MACRO18  # type:ignore
KEY_MACRO19 = e.KEY_MACRO19  # type:ignore
KEY_MACRO2 = e.KEY_MACRO2  # type:ignore
KEY_MACRO20 = e.KEY_MACRO20  # type:ignore
KEY_MACRO21 = e.KEY_MACRO21  # type:ignore
KEY_MACRO22 = e.KEY_MACRO22  # type:ignore
KEY_MACRO23 = e.KEY_MACRO23  # type:ignore
KEY_MACRO24 = e.KEY_MACRO24  # type:ignore
KEY_MACRO25 = e.KEY_MACRO25  # type:ignore
KEY_MACRO26 = e.KEY_MACRO26  # type:ignore
KEY_MACRO27 = e.KEY_MACRO27  # type:ignore
KEY_MACRO28 = e.KEY_MACRO28  # type:ignore
KEY_MACRO29 = e.KEY_MACRO29  # type:ignore
KEY_MACRO3 = e.KEY_MACRO3  # type:ignore
KEY_MACRO30 = e.KEY_MACRO30  # type:ignore
KEY_MACRO4 = e.KEY_MACRO4  # type:ignore
KEY_MACRO5 = e.KEY_MACRO5  # type:ignore
KEY_MACRO6 = e.KEY_MACRO6  # type:ignore
KEY_MACRO7 = e.KEY_MACRO7  # type:ignore
KEY_MACRO8 = e.KEY_MACRO8  # type:ignore
KEY_MACRO9 = e.KEY_MACRO9  # type:ignore
KEY_MACRO_PRESET1 = e.KEY_MACRO_PRESET1  # type:ignore
KEY_MACRO_PRESET2 = e.KEY_MACRO_PRESET2  # type:ignore
KEY_MACRO_PRESET3 = e.KEY_MACRO_PRESET3  # type:ignore
KEY_MACRO_PRESET_CYCLE = e.KEY_MACRO_PRESET_CYCLE  # type:ignore
KEY_MACRO_RECORD_START = e.KEY_MACRO_RECORD_START  # type:ignore
KEY_MACRO_RECORD_STOP = e.KEY_MACRO_RECORD_STOP  # type:ignore
KEY_MAIL = e.KEY_MAIL  # type:ignore
KEY_MAX = e.KEY_MAX  # type:ignore
KEY_MEDIA = e.KEY_MEDIA  # type:ignore
KEY_MEDIA_REPEAT = e.KEY_MEDIA_REPEAT  # type:ignore
KEY_MEDIA_TOP_MENU = e.KEY_MEDIA_TOP_MENU  # type:ignore
KEY_MEMO = e.KEY_MEMO  # type:ignore
KEY_MENU = e.KEY_MENU  # type:ignore
KEY_MESSENGER = e.KEY_MESSENGER  # type:ignore
KEY_MHP = e.KEY_MHP  # type:ignore
KEY_MICMUTE = e.KEY_MICMUTE  # type:ignore
KEY_MINUS = e.KEY_MINUS  # type:ignore
KEY_MIN_INTERESTING = e.KEY_MIN_INTERESTING  # type:ignore
KEY_MODE = e.KEY_MODE  # type:ignore
KEY_MOVE = e.KEY_MOVE  # type:ignore
KEY_MP3 = e.KEY_MP3  # type:ignore
KEY_MSDOS = e.KEY_MSDOS  # type:ignore
KEY_MUHENKAN = e.KEY_MUHENKAN  # type:ignore
KEY_MUTE = e.KEY_MUTE  # type:ignore
KEY_N = e.KEY_N  # type:ignore
KEY_NEW = e.KEY_NEW  # type:ignore
KEY_NEWS = e.KEY_NEWS  # type:ignore
KEY_NEXT = e.KEY_NEXT  # type:ignore
KEY_NEXTSONG = e.KEY_NEXTSONG  # type:ignore
KEY_NEXT_FAVORITE = e.KEY_NEXT_FAVORITE  # type:ignore
KEY_NOTIFICATION_CENTER = e.KEY_NOTIFICATION_CENTER  # type:ignore
KEY_NUMERIC_0 = e.KEY_NUMERIC_0  # type:ignore
KEY_NUMERIC_1 = e.KEY_NUMERIC_1  # type:ignore
KEY_NUMERIC_11 = e.KEY_NUMERIC_11  # type:ignore
KEY_NUMERIC_12 = e.KEY_NUMERIC_12  # type:ignore
KEY_NUMERIC_2 = e.KEY_NUMERIC_2  # type:ignore
KEY_NUMERIC_3 = e.KEY_NUMERIC_3  # type:ignore
KEY_NUMERIC_4 = e.KEY_NUMERIC_4  # type:ignore
KEY_NUMERIC_5 = e.KEY_NUMERIC_5  # type:ignore
KEY_NUMERIC_6 = e.KEY_NUMERIC_6  # type:ignore
KEY_NUMERIC_7 = e.KEY_NUMERIC_7  # type:ignore
KEY_NUMERIC_8 = e.KEY_NUMERIC_8  # type:ignore
KEY_NUMERIC_9 = e.KEY_NUMERIC_9  # type:ignore
KEY_NUMERIC_A = e.KEY_NUMERIC_A  # type:ignore
KEY_NUMERIC_B = e.KEY_NUMERIC_B  # type:ignore
KEY_NUMERIC_C = e.KEY_NUMERIC_C  # type:ignore
KEY_NUMERIC_D = e.KEY_NUMERIC_D  # type:ignore
KEY_NUMERIC_POUND = e.KEY_NUMERIC_POUND  # type:ignore
KEY_NUMERIC_STAR = e.KEY_NUMERIC_STAR  # type:ignore
KEY_NUMLOCK = e.KEY_NUMLOCK  # type:ignore
KEY_O = e.KEY_O  # type:ignore
KEY_OK = e.KEY_OK  # type:ignore
KEY_ONSCREEN_KEYBOARD = e.KEY_ONSCREEN_KEYBOARD  # type:ignore
KEY_OPEN = e.KEY_OPEN  # type:ignore
KEY_OPTION = e.KEY_OPTION  # type:ignore
KEY_P = e.KEY_P  # type:ignore
KEY_PAGEDOWN = e.KEY_PAGEDOWN  # type:ignore
KEY_PAGEUP = e.KEY_PAGEUP  # type:ignore
KEY_PASTE = e.KEY_PASTE  # type:ignore
KEY_PAUSE = e.KEY_PAUSE  # type:ignore
KEY_PAUSECD = e.KEY_PAUSECD  # type:ignore
KEY_PAUSE_RECORD = e.KEY_PAUSE_RECORD  # type:ignore
KEY_PC = e.KEY_PC  # type:ignore
KEY_PHONE = e.KEY_PHONE  # type:ignore
KEY_PICKUP_PHONE = e.KEY_PICKUP_PHONE  # type:ignore
KEY_PLAY = e.KEY_PLAY  # type:ignore
KEY_PLAYCD = e.KEY_PLAYCD  # type:ignore
KEY_PLAYER = e.KEY_PLAYER  # type:ignore
KEY_PLAYPAUSE = e.KEY_PLAYPAUSE  # type:ignore
KEY_POWER = e.KEY_POWER  # type:ignore
KEY_POWER2 = e.KEY_POWER2  # type:ignore
KEY_PRESENTATION = e.KEY_PRESENTATION  # type:ignore
KEY_PREVIOUS = e.KEY_PREVIOUS  # type:ignore
KEY_PREVIOUSSONG = e.KEY_PREVIOUSSONG  # type:ignore
KEY_PRINT = e.KEY_PRINT  # type:ignore
KEY_PRIVACY_SCREEN_TOGGLE = e.KEY_PRIVACY_SCREEN_TOGGLE  # type:ignore
KEY_PROG1 = e.KEY_PROG1  # type:ignore
KEY_PROG2 = e.KEY_PROG2  # type:ignore
KEY_PROG3 = e.KEY_PROG3  # type:ignore
KEY_PROG4 = e.KEY_PROG4  # type:ignore
KEY_PROGRAM = e.KEY_PROGRAM  # type:ignore
KEY_PROPS = e.KEY_PROPS  # type:ignore
KEY_PVR = e.KEY_PVR  # type:ignore
KEY_Q = e.KEY_Q  # type:ignore
KEY_QUESTION = e.KEY_QUESTION  # type:ignore
KEY_R = e.KEY_R  # type:ignore
KEY_RADIO = e.KEY_RADIO  # type:ignore
KEY_RECORD = e.KEY_RECORD  # type:ignore
KEY_RED = e.KEY_RED  # type:ignore
KEY_REDO = e.KEY_REDO  # type:ignore
KEY_REFRESH = e.KEY_REFRESH  # type:ignore
KEY_REPLY = e.KEY_REPLY  # type:ignore
KEY_RESERVED = e.KEY_RESERVED  # type:ignore
KEY_RESTART = e.KEY_RESTART  # type:ignore
KEY_REWIND = e.KEY_REWIND  # type:ignore
KEY_RFKILL = e.KEY_RFKILL  # type:ignore
KEY_RIGHT = e.KEY_RIGHT  # type:ignore
KEY_RIGHTALT = e.KEY_RIGHTALT  # type:ignore
KEY_RIGHTBRACE = e.KEY_RIGHTBRACE  # type:ignore
KEY_RIGHTCTRL = e.KEY_RIGHTCTRL  # type:ignore
KEY_RIGHTMETA = e.KEY_RIGHTMETA  # type:ignore
KEY_RIGHTSHIFT = e.KEY_RIGHTSHIFT  # type:ignore
KEY_RIGHT_DOWN = e.KEY_RIGHT_DOWN  # type:ignore
KEY_RIGHT_UP = e.KEY_RIGHT_UP  # type:ignore
KEY_RO = e.KEY_RO  # type:ignore
KEY_ROOT_MENU = e.KEY_ROOT_MENU  # type:ignore
KEY_ROTATE_DISPLAY = e.KEY_ROTATE_DISPLAY  # type:ignore
KEY_ROTATE_LOCK_TOGGLE = e.KEY_ROTATE_LOCK_TOGGLE  # type:ignore
KEY_S = e.KEY_S  # type:ignore
KEY_SAT = e.KEY_SAT  # type:ignore
KEY_SAT2 = e.KEY_SAT2  # type:ignore
KEY_SAVE = e.KEY_SAVE  # type:ignore
KEY_SCALE = e.KEY_SCALE  # type:ignore
KEY_SCREEN = e.KEY_SCREEN  # type:ignore
KEY_SCREENLOCK = e.KEY_SCREENLOCK  # type:ignore
KEY_SCREENSAVER = e.KEY_SCREENSAVER  # type:ignore
KEY_SCROLLDOWN = e.KEY_SCROLLDOWN  # type:ignore
KEY_SCROLLLOCK = e.KEY_SCROLLLOCK  # type:ignore
KEY_SCROLLUP = e.KEY_SCROLLUP  # type:ignore
KEY_SEARCH = e.KEY_SEARCH  # type:ignore
KEY_SELECT = e.KEY_SELECT  # type:ignore
KEY_SELECTIVE_SCREENSHOT = e.KEY_SELECTIVE_SCREENSHOT  # type:ignore
KEY_SEMICOLON = e.KEY_SEMICOLON  # type:ignore
KEY_SEND = e.KEY_SEND  # type:ignore
KEY_SENDFILE = e.KEY_SENDFILE  # type:ignore
KEY_SETUP = e.KEY_SETUP  # type:ignore
KEY_SHOP = e.KEY_SHOP  # type:ignore
KEY_SHUFFLE = e.KEY_SHUFFLE  # type:ignore
KEY_SLASH = e.KEY_SLASH  # type:ignore
KEY_SLEEP = e.KEY_SLEEP  # type:ignore
KEY_SLOW = e.KEY_SLOW  # type:ignore
KEY_SLOWREVERSE = e.KEY_SLOWREVERSE  # type:ignore
KEY_SOUND = e.KEY_SOUND  # type:ignore
KEY_SPACE = e.KEY_SPACE  # type:ignore
KEY_SPELLCHECK = e.KEY_SPELLCHECK  # type:ignore
KEY_SPORT = e.KEY_SPORT  # type:ignore
KEY_SPREADSHEET = e.KEY_SPREADSHEET  # type:ignore
KEY_STOP = e.KEY_STOP  # type:ignore
KEY_STOPCD = e.KEY_STOPCD  # type:ignore
KEY_STOP_RECORD = e.KEY_STOP_RECORD  # type:ignore
KEY_SUBTITLE = e.KEY_SUBTITLE  # type:ignore
KEY_SUSPEND = e.KEY_SUSPEND  # type:ignore
KEY_SWITCHVIDEOMODE = e.KEY_SWITCHVIDEOMODE  # type:ignore
KEY_SYSRQ = e.KEY_SYSRQ  # type:ignore
KEY_T = e.KEY_T  # type:ignore
KEY_TAB = e.KEY_TAB  # type:ignore
KEY_TAPE = e.KEY_TAPE  # type:ignore
KEY_TASKMANAGER = e.KEY_TASKMANAGER  # type:ignore
KEY_TEEN = e.KEY_TEEN  # type:ignore
KEY_TEXT = e.KEY_TEXT  # type:ignore
KEY_TIME = e.KEY_TIME  # type:ignore
KEY_TITLE = e.KEY_TITLE  # type:ignore
KEY_TOUCHPAD_OFF = e.KEY_TOUCHPAD_OFF  # type:ignore
KEY_TOUCHPAD_ON = e.KEY_TOUCHPAD_ON  # type:ignore
KEY_TOUCHPAD_TOGGLE = e.KEY_TOUCHPAD_TOGGLE  # type:ignore
KEY_TUNER = e.KEY_TUNER  # type:ignore
KEY_TV = e.KEY_TV  # type:ignore
KEY_TV2 = e.KEY_TV2  # type:ignore
KEY_TWEN = e.KEY_TWEN  # type:ignore
KEY_U = e.KEY_U  # type:ignore
KEY_UNDO = e.KEY_UNDO  # type:ignore
KEY_UNKNOWN = e.KEY_UNKNOWN  # type:ignore
KEY_UNMUTE = e.KEY_UNMUTE  # type:ignore
KEY_UP = e.KEY_UP  # type:ignore
KEY_UWB = e.KEY_UWB  # type:ignore
KEY_V = e.KEY_V  # type:ignore
KEY_VCR = e.KEY_VCR  # type:ignore
KEY_VCR2 = e.KEY_VCR2  # type:ignore
KEY_VENDOR = e.KEY_VENDOR  # type:ignore
KEY_VIDEO = e.KEY_VIDEO  # type:ignore
KEY_VIDEOPHONE = e.KEY_VIDEOPHONE  # type:ignore
KEY_VIDEO_NEXT = e.KEY_VIDEO_NEXT  # type:ignore
KEY_VIDEO_PREV = e.KEY_VIDEO_PREV  # type:ignore
KEY_VOD = e.KEY_VOD  # type:ignore
KEY_VOICECOMMAND = e.KEY_VOICECOMMAND  # type:ignore
KEY_VOICEMAIL = e.KEY_VOICEMAIL  # type:ignore
KEY_VOLUMEDOWN = e.KEY_VOLUMEDOWN  # type:ignore
KEY_VOLUMEUP = e.KEY_VOLUMEUP  # type:ignore
KEY_W = e.KEY_W  # type:ignore
KEY_WAKEUP = e.KEY_WAKEUP  # type:ignore
KEY_WIMAX = e.KEY_WIMAX  # type:ignore
KEY_WLAN = e.KEY_WLAN  # type:ignore
KEY_WORDPROCESSOR = e.KEY_WORDPROCESSOR  # type:ignore
KEY_WPS_BUTTON = e.KEY_WPS_BUTTON  # type:ignore
KEY_WWAN = e.KEY_WWAN  # type:ignore
KEY_WWW = e.KEY_WWW  # type:ignore
KEY_X = e.KEY_X  # type:ignore
KEY_XFER = e.KEY_XFER  # type:ignore
KEY_Y = e.KEY_Y  # type:ignore
KEY_YELLOW = e.KEY_YELLOW  # type:ignore
KEY_YEN = e.KEY_YEN  # type:ignore
KEY_Z = e.KEY_Z  # type:ignore
KEY_ZENKAKUHANKAKU = e.KEY_ZENKAKUHANKAKU  # type:ignore
KEY_ZOOM = e.KEY_ZOOM  # type:ignore
KEY_ZOOMIN = e.KEY_ZOOMIN  # type:ignore
KEY_ZOOMOUT = e.KEY_ZOOMOUT  # type:ignore
KEY_ZOOMRESET = e.KEY_ZOOMRESET  # type:ignore



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wayremap']

package_data = \
{'': ['*']}

install_requires = \
['evdev>=1.4.0,<2.0.0', 'i3ipc>=2.2.1,<3.0.0', 'python-uinput>=0.11.2,<0.12.0']

setup_kwargs = {
    'name': 'wayremap',
    'version': '0.0.8',
    'description': 'A dynamic keyboard remapper for Wayland.',
    'long_description': "[![test](https://github.com/acro5piano/wayremap/actions/workflows/test.yml/badge.svg)](https://github.com/acro5piano/wayremap/actions/workflows/test.yml)\n[![PyPI version](https://badge.fury.io/py/wayremap.svg)](https://badge.fury.io/py/wayremap)\n\n# wayremap\n\nDynamic keyboard remapper for Wayland.\n\nIt works on both X Window Manager and Wayland, but focused on Wayland as it intercepts evdev input and require root permission.\n\n# Motivation\n\nWayland and Sway is awesome. It brings lots of benefit to Linux desktop environment.\n\nWhen I was using X desktop envionment, there is an awesome tool called `xremap` which remap keys **based on current focused application**.\n\nhttps://github.com/k0kubun/xremap\n\nI was looking for something similar to `xremap` for Wayland, but not found, so I decided to create on my own.\n\n# Install\n\n```bash\nsudo pip install wayremap\n\n# For beta version\nsudo pip3 install git+https://github.com/acro5piano/wayremap\n\n```\n\n# Run\n\nFor Wayland security model, we have to do execute key remapping as root.\n\nSimply write your own service and run it as python script:\n\n```python\n # /etc/wayremap.config.py\n\nfrom wayremap.config import WayremapConfig, Binding\nfrom wayremap.main import run\nfrom wayremap import ecodes as e\nimport uinput as k\n\nwayremap_config = WayremapConfig(\n    # Filter applications which remap will be applied\n    applications=[\n        'Chromium',\n        'Brave-browser',\n        'Leafpad',\n        'firefoxdeveloperedition',\n    ],\n    bindings=[\n        # To see all available binding keys, please see\n        # https://github.com/acro5piano/wayremap/blob/06d27c9bb86b766d7fd1e4230f3a16827785519e/wayremap/ecodes.py\n        # modifier keys are `KEY_LEFTCTRL` or `KEY_LEFTALT`, or both. Neither `shift` nor `super` is not implemented yet.\n\n        # Emacs-like key binding\n        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_A],\n                [[k.KEY_LEFTCTRL, k.KEY_HOME]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_E],\n                [[k.KEY_LEFTCTRL, k.KEY_END]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_LEFTALT, e.KEY_H],\n                [[k.KEY_LEFTCTRL, k.KEY_BACKSPACE]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_F], [[k.KEY_RIGHT]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_B], [[k.KEY_LEFT]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_P], [[k.KEY_UP]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_N], [[k.KEY_DOWN]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_K],\n                [[k.KEY_LEFTSHIFT, k.KEY_END], [k.KEY_LEFTCTRL, k.KEY_X]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_A], [[k.KEY_HOME]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_E], [[k.KEY_END]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_Y], [[k.KEY_LEFTCTRL, k.KEY_V]]),\n        Binding([e.KEY_LEFTALT, e.KEY_F], [[k.KEY_LEFTCTRL, k.KEY_RIGHT]]),\n        Binding([e.KEY_LEFTALT, e.KEY_B], [[k.KEY_LEFTCTRL, k.KEY_LEFT]]),\n        Binding([e.KEY_LEFTALT, e.KEY_D], [[k.KEY_LEFTCTRL, k.KEY_DELETE]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_H], [[k.KEY_BACKSPACE]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_D], [[k.KEY_DELETE]]),\n        Binding([e.KEY_LEFTCTRL, e.KEY_S], [[k.KEY_LEFTCTRL, k.KEY_F]]),\n\n\n        # OSX-like key binding\n        Binding([e.KEY_LEFTALT, e.KEY_A], [[k.KEY_LEFTCTRL, k.KEY_A]]),\n        Binding([e.KEY_LEFTALT, e.KEY_C], [[k.KEY_LEFTCTRL, k.KEY_C]]),\n        Binding([e.KEY_LEFTALT, e.KEY_V], [[k.KEY_LEFTCTRL, k.KEY_V]]),\n\n        # Slack helm!\n        Binding([e.KEY_LEFTALT, e.KEY_X], [[k.KEY_LEFTCTRL, k.KEY_K]]),\n    ])\n\n# Finally, run wayremap.\n# Note that `'/dev/input/event4'` varies among system.\nrun(wayremap_config, '/dev/input/event4')\n\n```\n\nAnd then\n\n```\nsudo modprobe uinput\nsudo python /opt/wayremap.py\n```\n\n# Enable wayremap as a systemd service\n\n```bash\necho uinput | sudo tee /etc/modules-load.d/wayremap.conf # Add uinput to auto-loaded linux modules\nsudo vim /etc/wayremap.config.py # Edit your config\nsudo cp ./systemd/wayremap.service /etc/systemd/system/wayremap.service\nsudo systemctl enable wayremap\nsudo reboot\n```\n\n# Known bugs\n\n- ~~`3` is pressed when changing focused window~~ → Fixed now\n- Key repeating become slow while switching focused windowd\n\n# Roadmap\n\n- [x] Enable to run wihtout Sway.\n- [ ] Support `shift` and `super` keys too.\n- [ ] Packaging for Arch Linux, Debian, Fedora, etc.\n- [ ] Enable to load per-application config.\n- [ ] Re-write in Rust for better performance.\n",
    'author': 'Kay Gosho',
    'author_email': 'ketsume0211@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/acro5piano/wayremap',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import time
import sys
import evdev
import uinput
import os
from i3ipc import Connection, Event
from threading import Thread
import traceback

from wayremap import config
from wayremap import constants


def is_pressed(value: int) -> bool:
    return value == 1 or value == 2


def list_devices():
    devices = [evdev.InputDevice(path) for path in evdev.list_devices()]
    for device in devices:
        print(device.path, device.name, device.phys)


def find_sway_ipc_path() -> str:
    for root, _, files in os.walk('/run/user/'):
        for file in files:
            if 'sway-ipc' in file:
                return os.path.join(root, file)
    raise Exception('Cannot find sway socket under /run/user/')


is_remap_enabled = True


def subscribe_sway(apps: list[str]):
    if apps is None or len(apps) == 0:
        print(
            "Warning: No applications specified to config. Applying remap to all applications."
        )
        return

    def on_window_focus(i3, _):
        global is_remap_enabled
        focused = i3.get_tree().find_focused()
        if not focused:  # some applications like bemenu, this check is needed.
            return
        app_name = focused.app_id or focused.window_class
        is_remap_enabled = app_name in apps
        print('Remap {} for {}'.format(is_remap_enabled, app_name))

    try:
        sway = Connection(find_sway_ipc_path())
        sway.on(Event.WINDOW_FOCUS, on_window_focus)
        sway.main()
    except:
        print(
            "Warning: Couldn't connect to Sway and  fallback to apply all applications. Is sway running?"
        )
        print(traceback.format_exc())


def remap(bindings: list[config.Binding], path: str):
    global is_remap_enabled
    real_input = evdev.InputDevice(path)
    print('using device:', real_input)

    is_ctrl = False
    is_alt = False

    try:
        with uinput.Device(constants.ALL_KEYS) as virtual_uinput:
            time.sleep(1)  # Important delay
            real_input.grab()

            for event in real_input.read_loop():
                if event.type == constants.EV_KEY:
                    if event.code in constants.CTRL_KEYS:
                        is_ctrl = is_pressed(event.value)
                    if event.code in constants.ALT_KEYS:
                        is_alt = is_pressed(event.value)
                    handled = False
                    for binding in bindings:
                        pass_ctrl = is_ctrl and binding.only_ctrl()
                        pass_alt = is_alt and binding.only_alt()
                        pass_ctrl_alt = is_ctrl and is_alt and binding.require_ctrl_alt(
                        )
                        if (is_remap_enabled and not handled
                                and (pass_ctrl or pass_alt or pass_ctrl_alt)
                                and event.code == binding.get_remap_keycode()
                                and is_pressed(event.value)):
                            handled = True
                            virtual_uinput.emit(uinput.KEY_CAPSLOCK, 0)
                            virtual_uinput.emit(uinput.KEY_LEFTALT, 0)
                            for key_combo in binding.to:
                                virtual_uinput.emit_combo(key_combo)
                            if is_ctrl:
                                virtual_uinput.emit(uinput.KEY_CAPSLOCK, 1)
                            if is_alt:
                                virtual_uinput.emit(uinput.KEY_LEFTALT, 1)
                    if not handled:
                        virtual_uinput.emit((0x01, event.code), event.value)
    except KeyboardInterrupt:
        real_input.ungrab()
        print(traceback.format_exc())
        sys.exit(0)
    except OSError:
        print(
            "Error:\n  Failed to get device. Did you forget to run `sudo modprobe uinput`?"
        )
        print(traceback.format_exc())


def run(config: config.WayremapConfig, path: str):
    threads = list()

    thread_remap = Thread(target=remap, args=(config.bindings, path))
    threads.append(thread_remap)
    thread_remap.start()

    thread_subscribe_sway = Thread(target=subscribe_sway,
                                   args=(config.applications, ))
    threads.append(thread_subscribe_sway)
    thread_subscribe_sway.start()

    for t in threads:
        t.join()


if __name__ == '__main__':
    # list_devices()
    run(config.example_config, '/dev/input/event4')
